<?php

namespace Drp\UserManagement;

use Drp\ProjectHelpers\ProjectHelpersServiceProvider;
use Drp\UserManagement\Contracts\ManagedUser;
use Drp\UserManagement\Contracts\TokenGenerator;
use Drp\UserManagement\Http\Middleware\Approved;
use Drp\UserManagement\Http\Middleware\SetupComplete;
use Drp\UserManagement\Policies\UsersPolicy;
use Illuminate\Support\Facades\Gate;
use Illuminate\Support\ServiceProvider;
use Illuminate\Support\Facades\Route;

/**
 * Class UserManagementServiceProvider
 * @package Drp\SimpleCms
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class UserManagementServiceProvider extends ServiceProvider
{
    /**
     * Perform post-registration booting of services.
     *
     * @return void
     */
    public function boot()
    {
        $this->loadRoutesFrom(__DIR__ . '/../routes/auth.php');
        $this->loadRoutesFrom(__DIR__ . '/../routes/admin.php');
        $this->loadViewsFrom(__DIR__ . "/../resources/views", 'userManagement');

        $this->publishes([
            __DIR__ . '/../resources/views' => base_path('resources/views/vendor/userManagement'),
        ], 'views');

        $this->publishes([
            __DIR__ . '/../config/user_management.php' => config_path('user_management.php'),
        ], 'config');


        $this->loadTranslationsFrom(__DIR__ . '/../resources/lang', 'userManagement');

        $this->publishes([
            __DIR__ . '/../resources/lang' => resource_path('lang/vendor/userManagement'),
        ]);

        if (!class_exists('CreateUsersTable')) {
            $this->publishes(
                [
                    __DIR__ . '/../database/migrations/create_users_table.php.stub' =>
                        database_path('migrations/' . date('Y_m_d_His', time()) . '_create_users_table.php'),
                ],
                'migrations'
            );
        }

        $this->registerPolicies();
        $this->registerMiddleware();
    }

    /**
     * Register bindings in the container.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfigFrom(__DIR__ . '/../config/user_management.php', 'user_management');

        $this->app->register(ProjectHelpersServiceProvider::class);

        $this->app->bind(\Drp\UserManagement\Contracts\ManagedUser::class, config('user_management.model'));

        $this->app->bind(TokenGenerator::class, RandomTokenGenerator::class);
    }

    /**
     * Register the application's policies.
     *
     * @return void
     */
    public function registerPolicies()
    {
        Gate::policy(ManagedUser::class, UsersPolicy::class);
    }


    /**
     * Register the packages middleware.
     * @return void
     */
    protected function registerMiddleware()
    {
        Route::aliasMiddleware('approved', Approved::class);
        Route::aliasMiddleware('setupComplete', SetupComplete::class);
    }
}
